unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Registry, ComCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    OpenDialog1: TOpenDialog;
    StatusBar1: TStatusBar;
    ProgressBar1: TProgressBar;
    Timer1: TTimer;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    RichEdit1: TRichEdit;
    CheckBox1: TCheckBox;
    Button6: TButton;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);

  private
   LoaderFile: TMemoryStream;
   Procedure FillMemo;

  public
   TackTimer: Boolean;
   LPTBase: Word;
   LPTDataByte: Byte;
   function DetectDeviceOnLPT: Boolean;
   procedure MakeResetBoard;
   Procedure MakeSPIActive;
   Procedure DisableSPI;
   Procedure SRByte(var data: byte);
   Procedure CSH;
   Procedure WaitToTack;

  end;

const
 DefaultLPT = 'DefLPT';
 DefaultDir = 'BootFile';

//Instruction Name   Instruction Format   Operation
WREN = 6;      //00000110                           //Set Write Enable Latch
WRDI = 4;      //00000100                           //Reset Write Enable Latch
RDSR = 5;      //00000101                           //Read Status Register
WRSR = 1;      //00000001                           //Write Status Register
READ = 3;      //00000011                           //Read Data from Memory Array
WRTE = 2;      //00000010                           //Write Data to Memory Array


var
  Form1: TForm1;

implementation

uses Unit2;

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
begin
Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
   reg:TRegistry;
begin
  TackTimer:=False;
  LPTBase:=$FFFF;
  reg:=TRegistry.Create;
  reg.RootKey:=HKEY_LOCAL_MACHINE;
  reg.OpenKey('Software\VFX\LPTtoSPI',True);
  if reg.ValueExists(DefaultLPT) then
       LPTBase:=reg.ReadInteger(DefaultLPT)
  else reg.WriteInteger(DefaultLPT, LPTBase);

  if reg.ValueExists(DefaultDir) then
   begin
    OpenDialog1.InitialDir:=ExtractFilePath(reg.ReadString(DefaultDir));
    OpenDialog1.FileName:=reg.ReadString(DefaultDir);
   end
  else reg.WriteString(DefaultDir, OpenDialog1.FileName);
  reg.CloseKey;
  reg.Free;


  LoaderFile:=TmemoryStream.Create;
  Button2.Click;




end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  reg:TRegistry;
begin
  reg:=TRegistry.Create;
  reg.RootKey:=HKEY_LOCAL_MACHINE;
  reg.OpenKey('Software\VFX\LPTtoSPI',True);
  reg.WriteString(DefaultDir, OpenDialog1.FileName);
  reg.WriteInteger(DefaultLPT, LPTBase);
  reg.CloseKey;
  reg.Free;

  LoaderFile.Destroy;

end;

function TForm1.DetectDeviceOnLPT: Boolean;
//detektalas alatt SPI busz tre-state lesz
begin
 TackTimer:=False;
 Timer1.Enabled:=True;
 asm
   push edx
   push edi

   mov di,$3BC
   call  @detect
   jz   @vege1

   mov di,$278
   call  @detect
   jz   @vege1

   mov di,$378
   call  @detect
   jz   @vege1

   mov di,$FFFF

   jmp  @vege1

@var:
  cmp byte ptr [ebx+TackTimer],0
  jnz @var
  mov byte ptr [ebx+TackTimer],0
  ret

@detect:
   mov dx,di
   mov al,$FC  //00
   out dx,al
   call WaitToTack
   inc dx
   in al,dx
   and eax,$A0   //mask PE @ BUSY - LPT 11,12
   cmp al,$80
   jnz     @Noport

   mov al,$FD     //01
   mov dx,di
   out dx,al
   call WaitToTack
   inc dx
   in al,dx
   and eax,$A0   //mask PE @ BUSY - LPT 11,12
   cmp al,$A0
   jnz     @Noport

   mov al,$FE     //02
   mov dx,di
   out dx,al
   call WaitToTack
   inc dx
   in al,dx
   and eax,$A0   //mask PE @ BUSY - LPT 11,12
   cmp al,$00
   jnz     @Noport

   mov al,$FF     //03
   mov dx,di
   out dx,al
   call WaitToTack
   inc dx
   in al,dx
   and eax,$A0   //mask PE @ BUSY - LPT 11,12
   cmp al,$20
@Noport:
    ret

@vege1:
   cmp di,$FFFF
   mov eax,False
   jz @kihagy
   mov eax,True
@kihagy:
   mov Result,al
   mov [ebx+LPTBase],di
   mov al,$EF                     //SCK = L
   mov [ebx+LPTDataByte],al
   pop edi
   pop edx
 end;
  Timer1.Enabled:=False;

end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
 TackTimer:=True;
end;



procedure TForm1.Button2Click(Sender: TObject);
begin
 if DetectDeviceOnLPT then
  begin
   Button3.Enabled:=True;
   StatusBar1.SimpleText:='Device on LPT: 0x'+IntToHex(LPTBase,3);
  end
 else
  begin
  Button3.Enabled:=False;
   StatusBar1.SimpleText:='Device on LPT: Not Found';
   Application.MessageBox(
        'SPI Download Device Not Found','Warning...',
        MB_OK);
  end;
end;




procedure TForm1.Button3Click(Sender: TObject);
begin
 if OpenDialog1.Execute then
  begin
   LoaderFile.LoadFromFile(OpenDialog1.FileName);
   StatusBar1.SimpleText:=OpenDialog1.FileName+' loaded';
   Button5.Enabled:=True;
   FillMemo;
  end;

end;

procedure TForm1.MakeResetBoard;
begin
 TackTimer:=False;
 Timer1.Enabled:=True;
 asm
 push edx
 mov dx,[ebx+LPTBase]
 mov al,[ebx+LPTDataByte]
 and al,$7B                     //LPT9 = L, LPT4 = L RESET aktiv & OE enabled
 mov [ebx+LPTDataByte],al
 out dx,al

 mov al,10                      //10ms varakozas a reset utan
@var1xa:
 call WaitToTack
 dec al
 jnz @var1xa
 pop edx
 end;
 Timer1.Enabled:=False;
end;

Procedure TForm1.MakeSPIActive;
begin
 MakeResetBoard;
 TackTimer:=False;
 Timer1.Enabled:=True;
 asm
 push edx
 mov dx,[ebx+LPTBase]
 mov al,[ebx+LPTDataByte]
 and al,$73                     //LPT9 = L, LPT4 = L RESET aktiv & OE enabled
                                //LPT5 = L SPI OE enabled
 mov [ebx+LPTDataByte],al
 out dx,al

 mov al,5                      //5ms SPI OE = Enabled utan
@var1xb:
 call WaitToTack
  dec al
  jnz @var1xb
 pop edx
 end;
 Timer1.Enabled:=False;
end;


Procedure TForm1.SRByte(var data: byte);
begin
 TackTimer:=False;
 Timer1.Enabled:=True;
 asm
  push edx
  push ecx
  push edi

  mov edi,data
  mov ch,[edi]
  mov dx,[ebx+LPTBase]
  mov al,[ebx+LPTDataByte]
  and al,$AF                     //CS =L, SCK = L
  out dx,al
  mov cl,8                       //8 bitet rotalunk

@rota:
  or  al,32                      //Data bit az 5. bit = MOSI
  rcl ah,2
  rcl ch,1
  jc  @bitok
  and al,$DF
@bitok:
  out dx,al
  call WaitToTack

  or  al,16                      //SCK =H
  out dx,al
  call WaitToTack

  xchg al,ah
  inc dx
  in  al,dx                      //ah
  dec dx
  xchg al,ah

  and  al,$EF                    //SCK =L
  out dx,al

  dec  cl
  jnz  @rota

  rcl ah,2
  rcl ch,1

  mov [ebx+LPTDataByte],al
  jmp @kimarad2

@kimarad2:
  mov [edi],ch
  pop edi
  pop ecx
  pop edx
 end;
 Timer1.Enabled:=False;
end;


Procedure TForm1.CSH;
begin
 TackTimer:=False;
 Timer1.Enabled:=True;
 asm
 push edx
 mov dx,[ebx+LPTBase]
 mov al,[ebx+LPTDataByte]
 or  al,64                            //CS = H
 out dx,al
 mov [ebx+LPTDataByte],al
 call WaitToTack
 pop edx
 end;
 Timer1.Enabled:=False;
end;




procedure TForm1.Button4Click(Sender: TObject);
begin
 MakeResetBoard;
 DisableSPI;
end;

procedure TForm1.Button5Click(Sender: TObject);
var
 SPIData, TempData: byte;
 SPIAddress: DWord;
 SPIError,I,ByteCount: DWord;
 RDSt: TMemoryStream;

begin
   RDSt:=TMemoryStream.Create;

   LoaderFile.LoadFromFile(OpenDialog1.FileName);
   FillMemo;
   StatusBar1.SimpleText:='Loader file reloaded...';
   SPIAddress:=0;
   if LoaderFile.Size>65536 then ByteCount:=65536 else
                                 ByteCount:=LoaderFile.Size;

   LoaderFile.Position:=SPIAddress;
   ProgressBar1.Min:=SPIAddress;
   ProgressBar1.Max:=ByteCount;
   ProgressBar1.Position:=SPIAddress;
   ProgressBar1.step:=32;

   StatusBar1.SimpleText:='Programming...';


 repeat
  application.processmessages;
  MakeSPIActive;
  SPIData:=WREN;
  SRByte(SPIData);                     //Send Write Enable Command
  CSH;

  MakeSPIActive;
  SPIData:=WRTE;                       //Send Write byte Command
  SRByte(SPIData);                     //Send Command

  SPIData:=SPIAddress div 256;
  SRByte(SPIData);
  SPIData:=SPIAddress mod 256;
  SRByte(SPIData);

  LoaderFile.Read(SPIData,1);
  SRByte(SPIData);
  CSH;                                 //Start 1 byte to program
  SPIAddress:=SPIAddress+1;

  repeat
   application.processmessages;
   MakeSPIActive;
   SPIData:=RDSR;                       //Send Read Status Register Command
   SRByte(SPIData);                     //Send Command
   SRByte(SPIData);
   CSH;
  until ((SPIData and 1)=0);

   if ((SPIAddress mod 32)=0) then ProgressBar1.StepIt;

 until (SPIAddress=ByteCount);


 MakeSPIActive;
 SPIData:=WRDI;
 SRByte(SPIData);                     //Send Write Disable Command
 CSH;

 if CheckBox1.Checked then
 begin

  StatusBar1.SimpleText:='Verifing...';
  SPIAddress:=0;
  ProgressBar1.Position:=SPIAddress;
  ProgressBar1.step:=1;

  MakeSPIActive;
  SPIData:=READ;                       //Send Read Command
  SRByte(SPIData);

  SPIData:=SPIAddress div 256;
  SRByte(SPIData);
  SPIData:=SPIAddress mod 256;
  SRByte(SPIData);

  repeat
   application.processmessages;
   SPIAddress:=SPIAddress+1;
   SRByte(SPIData);
   RDSt.Write(SPIData,1);
   ProgressBar1.StepIt;
  until (SPIAddress=ByteCount);

  CSH;                                 //Stop read all bytes

  StatusBar1.SimpleText:='EEPROM Read done...';
  LoaderFile.Position:=0;
  RDSt.Position:=0;
  SPIError:=0;
  for I:=0 to ByteCount do
   begin
     LoaderFile.Read(SPIData,1);
     RDSt.Read(TempData,1);
     if TempData<>SPIData then SPIError:=SPIError+1;
   end;

  if SPIError=0 then
   begin
     StatusBar1.SimpleText:='No error, EEPROM Write done';
     DisableSPI;
     ProgressBar1.Position:=ProgressBar1.Max;
   end
  else
   begin
     StatusBar1.SimpleText:='Verify error. '+IntToStr(SPIError)+' error(s) found.';
     Application.MessageBox('Verify error','Error...',MB_OK);
   end;

  end else   //CheckBox1 = Verify needed
  begin     //CheckBox1 = Verify not needed
     StatusBar1.SimpleText:='Unchecked EEPROM write done.';
     DisableSPI;
     ProgressBar1.Position:=ProgressBar1.Max;
  end;
  RDSt.Destroy;
end;


Procedure TForm1.FillMemo;
var
 Address,I: DWord;
 s:string;
 data:byte;
 begin
  LoaderFile.Position:=0;
  Address:=0;
  s:='';
  RichEdit1.Lines.Clear;

  repeat
   i:=0;
   s:=IntToHex(Address,4)+' ';
   repeat
    LoaderFile.Read(data,1);
    s:=s+' '+IntToHex(data,2);
    Address:=Address+1;
    I:=I+1;
   until (LoaderFile.Position=LoaderFile.Size) or (I=16);
   RichEdit1.Lines.Add(s);
  until (LoaderFile.Position=LoaderFile.Size);
 end;


 Procedure Tform1.DisableSPI;
 begin
  asm
   push edx
   mov dx,[ebx+LPTBase]
   mov al,[ebx+LPTDataByte]
   or al,$8C                     //SPI & RESET OE Disabled
   mov [ebx+LPTDataByte],al
   out dx,al
   pop edx
  end;
 end;


Procedure Tform1.WaitToTack; assembler;
asm
  push eax
   mov eax,186
@varxxx:

//  cmp byte ptr [ebx+TackTimer],0
//  jnz @varxxx
//  mov byte ptr [ebx+TackTimer],0
    nop
    nop
    dec eax
    jnz @varxxx


  pop eax
  ret
end;




procedure TForm1.Button6Click(Sender: TObject);
begin
 Form2.showmodal;
end;

end.
